define([
    'jquery',
    'uiRegistry',
    'Magento_Ui/js/grid/listing'
], function ($, registry, listing) {
    'use strict';

    var mixin = {
        applyAction: function (action, data) {
            // Call parent function if action is not pkp_mass_action
            if (action !== 'pkp_mass_action') {
                this._super();
                return;
            }

            $('body').trigger('processStart');
            // remove all messages
            $('#messages').remove();
            $('.mage-error').remove();
            var selections = this.getSelections();
            var dataSource = registry.get(this.provider);
            var items = dataSource.data.items;

            // Filter the items to get only the selected ones
            var selectedItems = items.filter(function (item) {
                return selections.selected.includes(item.entity_id.toString());
            });

            var payload = this.preparePayload(selectedItems);

            // Send payload to controller if valid
            if (payload) {
                this.sendPayloadToController(payload);
            } else {
                $('body').trigger('processStop');
            }
        },

        preparePayload: function (selectedItems) {
            var isValid = true;
            var payload = [];

            selectedItems.forEach(function (item) {
                var rowElement = $('[data-repeat-index="' + item._rowIndex + '"]');
                var pkpProductType = this.getAndValidateSelectValue(rowElement, '.pkp-select-product-grid');
                var pkpProductCount = this.getAndValidateSelectValue(rowElement, '.pkp-product-count-select');

                if (pkpProductType && pkpProductCount) {
                    payload.push({
                        'order_id': item.entity_id,
                        'pkp_product_type': pkpProductType,
                        'pkp_product_count': pkpProductCount
                    });
                } else {
                    isValid = false;
                }
            }, this);

            return isValid ? payload : null;
        },

        getAndValidateSelectValue: function (rowElement, selector) {
            var selectBox = rowElement.find(selector);
            var value = selectBox.val();

            if (!value) {
                // show error message on top of the grid
                selectBox.after('<div class="mage-error">' + $.mage.__('Please select a value') + '</div>');
                // make it red
                selectBox.addClass('admin__control-select-error');
                return null;
            }

            rowElement.find('.mage-error').remove();
            return value;
        },

        sendPayloadToController: function (payload) {
            var actionUrl = this.getActionUrl();

            var formData = new FormData();
            formData.append('form_key', window.FORM_KEY);
            formData.append('payload', JSON.stringify(payload));

            $.ajax({
                url: actionUrl,
                type: 'POST',
                data: formData,
                processData: false,
                contentType: false,
                success: function (response) {
                    // Handle success
                    if (response.success) {

                        $('main').prepend('<div id="messages"><div class="messages"><div class="message message-success success"><div data-ui-id="messages-message-success"> Orders set Successfully! </div></div></div></div>')
                        window.open(response.url, '_blank');

                    } else {
                        if (response.error) {
                            $('main').prepend('<div id="messages"><div class="messages"><div class="message message-error error"><div data-ui-id="messages-message-error">' + response.error + '</div></div></div></div>')
                        }

                    }
                    // scroll to messages
                    $('html, body').animate({
                        scrollTop: $("#messages").offset().top
                    }, 1000);
                    $('body').trigger('processStop');
                },
                error: function () {
                    // Handle error
                    $('body').trigger('processStop');
                    $('main').prepend('<div id="messages"><div class="messages"><div class="message message-error error"><div data-ui-id="messages-message-error">' + $.mage.__('An error occurred while processing the request. Please try again later.') + '</div></div></div></div>')
                    // scroll to messages
                    $('html, body').animate({
                        scrollTop: $("#messages").offset().top
                    }, 1000);
                }
            });
        },

        getActionUrl: function () {
            var actionUrl = '';
            this.actions._latestValue.forEach(function (action) {
                if (action.component === 'PkpMassAction') {
                    actionUrl = action.url;
                }
            });
            return actionUrl;
        }
    };

    return function (target) {
        return target.extend(mixin);
    };
});
